# Generated by roxygen2: do not edit by hand

export(animate)
export(animateOutput)
export(click_to_loop)
export(click_to_play)
export(insert_animate)
export(loop)
export(new_id)
export(rmd_animate)
import(R6)
import(httpuv)
importFrom(utils,tail)
